@@ -44,6 +44,7 @@ class ENV_VARIABLES:
     HOPSWORKS_PROJECT_ID_ENV_VAR = "HOPSWORKS_PROJECT_ID"
     HOPSWORKS_PROJECT_NAME_ENV_VAR = "HOPSWORKS_PROJECT_NAME"
     API_KEY_ENV_VAR = "API_KEY"
+    SERVING_API_KEY_ENV_VAR = "SERVING_API_KEY"
     REGION_NAME_ENV_VAR = "REGION_NAME"
     FLINK_CONF_DIR = "FLINK_CONF_DIR"
     FLINK_LIB_DIR = "FLINK_LIB_DIR"
@@ -170,25 +171,31 @@ class SPARK_CONFIG:
     SPARK_VECTOR = "vector"
     SPARK_SQL_CATALOG_IMPLEMENTATION = "spark.sql.catalogImplementation"
     SPARK_SQL_CATALOG_HIVE = "hive"
-    SPARK_JDBC_FORMAT= "jdbc"
-    SPARK_JDBC_URL= "url"
-    SPARK_JDBC_DBTABLE= "dbtable"
+    SPARK_JDBC_FORMAT = "jdbc"
+    SPARK_JDBC_URL = "url"
+    SPARK_JDBC_DBTABLE = "dbtable"
     SPARK_JDBC_USER = "user"
     SPARK_JDBC_PW = "password"
 
+
 class MODEL_SERVING:
     MODELS_DATASET = "Models"
     MODEL_SERVER_TENSORFLOW_SERVING = "TENSORFLOW_SERVING"
-    MODEL_SERVER_FLASK = "FLASK"
-    MODEL_SERVERS = [MODEL_SERVER_TENSORFLOW_SERVING, MODEL_SERVER_FLASK]
+    MODEL_SERVER_PYTHON = "PYTHON"
+    MODEL_SERVERS = [MODEL_SERVER_TENSORFLOW_SERVING, MODEL_SERVER_PYTHON]
     SERVING_TOOL_DEFAULT = "DEFAULT"
     SERVING_TOOL_KFSERVING = "KFSERVING"
     SERVING_TOOLS = [SERVING_TOOL_DEFAULT, SERVING_TOOL_KFSERVING]
+    INFERENCE_LOGGING_MODEL_INPUTS = "MODEL_INPUTS"
+    INFERENCE_LOGGING_PREDICTIONS = "PREDICTIONS"
+    INFERENCE_LOGGING_ALL = "ALL"
+    INFERENCE_LOGGING_MODES = [INFERENCE_LOGGING_MODEL_INPUTS, INFERENCE_LOGGING_PREDICTIONS, INFERENCE_LOGGING_ALL]
     SERVING_ACTION_START = "START"
     SERVING_ACTION_STOP = "STOP"
     SERVING_ACTIONS = [SERVING_ACTION_STOP, SERVING_ACTION_STOP]
     SERVING_START_OR_STOP_PATH_PARAM = "?action="
 
+
 class FEATURE_STORE:
     """
      Featurestore constants
@@ -232,8 +239,8 @@ class FEATURE_STORE:
     ]
     FEATURE_GROUP_INSERT_APPEND_MODE = "append"
     FEATURE_GROUP_INSERT_OVERWRITE_MODE = "overwrite"
-    FEATURESTORE_SUFFIX =  "_featurestore"
-    TRAINING_DATASETS_SUFFIX =  "_Training_Datasets"
+    FEATURESTORE_SUFFIX = "_featurestore"
+    TRAINING_DATASETS_SUFFIX = "_Training_Datasets"
     TRAINING_DATASET_TF_RECORD_SCHEMA_FILE_NAME = "tf_record_schema.txt"
     TF_RECORD_SCHEMA_FEATURE = "feature"
     TF_RECORD_SCHEMA_FEATURE_FIXED = "fixed_len"
@@ -252,7 +259,7 @@ class FEATURE_STORE:
     TF_RECORD_FLOAT_SPARK_TYPES = [SPARK_CONFIG.SPARK_FLOAT_TYPE, SPARK_CONFIG.SPARK_DECIMAL_TYPE,
                                    SPARK_CONFIG.SPARK_DOUBLE_TYPE]
     TF_RECORD_FLOAT_ARRAY_SPARK_TYPES = [SPARK_CONFIG.SPARK_ARRAY_FLOAT, SPARK_CONFIG.SPARK_ARRAY_DECIMAL,
-                                   SPARK_CONFIG.SPARK_ARRAY_DOUBLE, SPARK_CONFIG.SPARK_VECTOR]
+                                         SPARK_CONFIG.SPARK_ARRAY_DOUBLE, SPARK_CONFIG.SPARK_VECTOR]
     RECOGNIZED_TF_RECORD_TYPES = [SPARK_CONFIG.SPARK_VECTOR, SPARK_CONFIG.SPARK_ARRAY_BINARY,
                                   SPARK_CONFIG.SPARK_ARRAY_STRING, SPARK_CONFIG.SPARK_ARRAY_DECIMAL,
                                   SPARK_CONFIG.SPARK_ARRAY_DOUBLE, SPARK_CONFIG.SPARK_ARRAY_FLOAT,
@@ -262,8 +269,7 @@ class FEATURE_STORE:
                                   SPARK_CONFIG.SPARK_FLOAT_TYPE, SPARK_CONFIG.SPARK_LONG_TYPE,
                                   SPARK_CONFIG.SPARK_INT_TYPE, SPARK_CONFIG.SPARK_INTEGER_TYPE,
                                   SPARK_CONFIG.SPARK_ARRAY_BIGINT, SPARK_CONFIG.SPARK_BIGINT_TYPE,
-                                  SPARK_CONFIG.SPARK_ARRAY_INT
-                                  ]
+                                  SPARK_CONFIG.SPARK_ARRAY_INT]
     DATAFRAME_TYPE_SPARK = "spark"
     DATAFRAME_TYPE_NUMPY = "numpy"
     DATAFRAME_TYPE_PYTHON = "python"
@@ -283,6 +289,7 @@ class PETASTORM_CONFIG:
     SCHEMA = "schema"
     LIBHDFS = "libhdfs"
 
+
 class MYSQL_CONFIG:
     """ MYSQL string constants """
     MYSQL_DATA_TYPES = [
@@ -298,6 +305,7 @@ class MYSQL_CONFIG:
     MYSQL_VARCHAR_1000_TYPE = "VARCHAR(1000)"
     MYSQL_BLOB_TYPE = "BLOB"
 
+
 class HIVE_CONFIG:
     """
     Hive string constants
@@ -319,7 +327,7 @@ class REST_CONFIG:
     JSON_KEYSTOREPWD = "keyStorePwd"
     JSON_SCHEMA_CONTENTS = "contents"
 
-    JSON_TYPE="type"
+    JSON_TYPE = "type"
 
     JSON_FEATURESTORE_UPDATE_STATS_QUERY_PARAM = "updateStats"
     JSON_FEATURESTORE_UPDATE_METADATA_QUERY_PARAM = "updateMetadata"
@@ -338,8 +346,7 @@ class REST_CONFIG:
     JSON_FEATURESTORE_SETTINGS_JDBC_CONNECTOR_DTO_TYPE = "jdbcConnectorDtoType"
     JSON_FEATURESTORE_SETTINGS_JDBC_CONNECTOR_TYPE = "jdbcConnectorType"
     JSON_FEATURESTORE_SETTINGS_JDBC_CONNECTOR_ARGUMENTS_MAX_LEN = "jdbcStorageConnectorArgumentsMaxLength"
-    JSON_FEATURESTORE_SETTINGS_JDBC_CONNECTOR_CONNECTION_STRING_MAX_LEN = \
-        "jdbcStorageConnectorConnectionstringMaxLength"
+    JSON_FEATURESTORE_SETTINGS_JDBC_CONNECTOR_CONNECTION_STRING_MAX_LEN = "jdbcStorageConnectorConnectionstringMaxLength"
     JSON_FEATURESTORE_SETTINGS_ON_DEMAND_FEATUREGROUP_DTO_TYPE = "onDemandFeaturegroupDtoType"
     JSON_FEATURESTORE_SETTINGS_ON_DEMAND_FEATUREGROUP_SQL_QUERY_MAX_LEN = "onDemandFeaturegroupSqlQueryMaxLength"
     JSON_FEATURESTORE_SETTINGS_S3_CONNECTOR_DTO_TYPE = "s3ConnectorDtoType"
@@ -393,7 +400,6 @@ class REST_CONFIG:
     JSON_ONLINE_FEATUREGROUP_TABLE_ROWS = "tableRows"
     JSON_ONLINE_FEATUREGROUP_SIZE = "size"
 
-
     JSON_FEATURESTORE = "featurestore"
     JSON_FEATURESTORE_ID = "featurestoreId"
     JSON_FEATURESTORE_NAME = "featurestoreName"
@@ -469,11 +475,12 @@ class REST_CONFIG:
     HOPSWORKS_FEATUREGROUPS_SYNC_RESOURCE = "sync"
     HOPSWORKS_SERVING_RESOURCE = "serving"
     HOPSWORKS_INFERENCE_RESOURCE = "inference"
+    HOPSWORKS_MODEL_REGISTRY_RESOURCE = "modelregistries"
     HOPSWORKS_MODELS_RESOURCE = "models"
     HOPSWORKS_USERS_RESOURCE = "users"
     HOPSWORKS_ADMIN_RESOURCE = "admin"
     HOPSWORKS_FEATURESTORES_STORAGE_CONNECTORS_RESOURCE = "storageconnectors"
-    HOPSWORKS_ONLINE_FEATURESTORE_STORAGE_CONNECTOR_RESOURCE= "onlinefeaturestore"
+    HOPSWORKS_ONLINE_FEATURESTORE_STORAGE_CONNECTOR_RESOURCE = "onlinefeaturestore"
     HOPSWORKS_FEATURESTORE_TAGS_RESOURCE = "tags"
     HOPSWORKS_VARIABLES_RESOURCE = "variables"
     HOPSWORKS_ENDPOINT = "hopsworks_endpoint"
@@ -485,9 +492,12 @@ class REST_CONFIG:
     HOPSWORKS_SUBJECTS_RESOURCE = "subjects"
 
     HOPSWORKS_AS_SHARED = "asShared"
+    HOPSWORKS_SHARED = "shared"
     HOPSWORKS_PROJECT_RESOURCE = "project"
+    HOPSWORKS_USER_RESOURCE = "users"
     HOPSWORKS_PROJECT_INFO_RESOURCE = "getProjectInfo"
     HOPSWORKS_JOBS_RESOURCE = "jobs"
+    HOPSWORKS_SECRETS_RESOURCE = "secrets"
     HOPSWORKS_EXECUTIONS_RESOURCE = "executions"
     HOPSWORKS_DATASETS_RESOURCE = "dataset"
     HOPSWORKS_PROJECT_CREDENTIALS_RESOURCE = "credentials"
@@ -505,21 +515,35 @@ class REST_CONFIG:
 
     JWT_TOKEN = "token.jwt"
 
-    JSON_SERVING_STATUS = "status"
-    JSON_SERVING_ARTIFACT_PATH = "artifactPath"
-    JSON_SERVING_NAME = "name"
-    JSON_SERVING_CREATOR = "creator"
-    JSON_MODEL_SERVER = "modelServer"
-    JSON_SERVING_TOOL = "servingTool"
-    JSON_SERVING_MODEL_VERSION = "modelVersion"
-    JSON_SERVING_CREATED = "created"
-    JSON_SERVING_REQUESTED_INSTANCES = "requestedInstances"
-    JSON_SERVING_BATCHING_ENABLED = "batchingEnabled"
-    JSON_SERVING_AVAILABLE_INSTANCES = "availableInstances"
-    JSON_SERVING_KAFKA_TOPIC_DTO = "kafkaTopicDTO"
     JSON_SERVING_ID = "id"
+    JSON_SERVING_NAME = "name"
+    JSON_SERVING_MODEL_PATH = "modelPath"
+    JSON_SERVING_MODEL_NAME = "modelName"
+    JSON_SERVING_MODEL_VERSION = "modelVersion"
+    JSON_SERVING_MODEL_SERVER = "modelServer"
+    JSON_SERVING_TOOL = "servingTool"
+    JSON_SERVING_ARTIFACT_VERSION = "artifactVersion"
+    JSON_SERVING_PREDICTOR = "predictor"
+    JSON_SERVING_TRANSFORMER = "transformer"
+    JSON_SERVING_REQUESTED_INSTANCES = "requestedInstances"
+    JSON_SERVING_PREDICTOR_RESOURCE_CONFIG = "predictorResourceConfig"
+    JSON_SERVING_REQUESTED_TRANSFORMER_INSTANCES = "requestedTransformerInstances"
+    JSON_SERVING_BATCHING_ENABLED = "batchingEnabled"
+    JSON_SERVING_KAFKA_TOPIC_DTO = "kafkaTopicDTO"
     JSON_SERVING_CREATE_KAFKA_TOPIC = "CREATE"
     JSON_SERVING_DONT_CREATE_KAFKA_TOPIC = "NONE"
+    JSON_SERVING_INFERENCE_LOGGING = "inferenceLogging"
+    JSON_SERVING_CREATED = "created"
+    JSON_SERVING_CREATOR = "creator"
+    JSON_SERVING_REVISION = "revision"
+    JSON_SERVING_STATUS = "status"
+    JSON_SERVING_AVAILABLE_INSTANCES = "availableInstances"
+    JSON_SERVING_AVAILABLE_TRANSFORMER_INSTANCES = "availableTransformerInstances"
+    JSON_SERVING_EXTERNAL_IP = "externalIP"
+    JSON_SERVING_EXTERNAL_PORT = "externalPort"
+    JSON_SERVING_INTERNAL_IPS = "internalIPs"
+    JSON_SERVING_INTERNAL_PORT = "internalPort"
+    JSON_SERVING_INTERNAL_PATH = "internalPath"
 
     JSON_KAFKA_TOPIC_SCHEMA_VERSION = "schemaVersion"
     JSON_KAFKA_TOPIC_NAME = "name"
@@ -571,17 +595,21 @@ class S3_CONFIG:
     AWS_SECRET_ACCESS_KEY_ENV = "AWS_SECRET_ACCESS_KEY"
     AWS_SESSION_TOKEN_ENV = "AWS_SESSION_TOKEN"
 
+
 class AWS:
     DEFAULT_REGION = 'default'
     SECRETS_MANAGER = "secretsmanager"
     PARAMETER_STORE = "parameterstore"
 
+
 class LOCAL:
     LOCAL_STORE = "local"
 
+
 class XATTRS:
     XATTRS_PARAM_NAME = 'name'
 
+
 class ELASTICSEARCH_CONFIG:
     SSL_CONFIG = "es.net.ssl"
     NODES_WAN_ONLY = "es.nodes.wan.only"
