@@ -56,6 +56,31 @@ def _init_logger(exec_logdir, role=None, index=None):
 
     return logfile.replace(hdfs.project_path(), '')
 
+def get_model_registry_id(project_name):
+    model_registry_id = None
+    if project_name is None:
+        return hdfs.project_id()
+    else:
+        headers = {constants.HTTP_CONFIG.HTTP_CONTENT_TYPE: constants.HTTP_CONFIG.HTTP_APPLICATION_JSON}
+        resource_url = constants.DELIMITERS.SLASH_DELIMITER + \
+                       constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER + \
+                       constants.REST_CONFIG.HOPSWORKS_PROJECT_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER + \
+                       hdfs.project_id() + constants.DELIMITERS.SLASH_DELIMITER + \
+                       constants.REST_CONFIG.HOPSWORKS_MODEL_REGISTRY_RESOURCE
+
+        response = util.send_request('GET', resource_url, headers=headers)
+        response_object = response.json()
+        if response.status_code >= 400:
+            error_code, error_msg, user_msg = util._parse_rest_error(response_object)
+            raise RestAPIError("Could not get model registries (url: {}), server response: \n "
+                               "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}"
+                .format(resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
+        else:
+            for registry in response_object['items']:
+                if registry['name'] == project_name:
+                    return registry['id']
+            raise RestAPIError('Could not find model registry shared from project {}'.format(project_name))
+
 def log(log_entry):
     """
     Logs a string to the log file
@@ -222,6 +247,15 @@ def _cleanup(tensorboard, gpu_thread):
     finally:
         tensorboard._reset_global()
 
+    # Stop the gpu monitoring thread
+    try:
+        if gpu_thread is not None and gpu_thread.is_alive():
+            gpu_thread.do_run = False
+            gpu_thread.join(timeout = 20)
+    except Exception as err:
+        print('Exception occurred while stopping GPU monitoring thread: {}'.format(err))
+        pass
+
     # Close and logging fd and flush
     try:
         _close_logger()
@@ -229,13 +263,6 @@ def _cleanup(tensorboard, gpu_thread):
         print('Exception occurred while closing logger: {}'.format(err))
         pass
 
-    # Stop the gpu monitoring thread
-    try:
-        gpu_thread.do_run = False
-    except Exception as err:
-        print('Exception occurred while stopping GPU monitoring thread: {}'.format(err))
-        pass
-
 def _store_local_tensorboard(local_tb_path, hdfs_exec_logdir):
     """
 
@@ -455,7 +482,7 @@ def _attach_experiment_xattr(ml_id, json_data, op_type):
     else:
         return response_object
 
-def _attach_model_xattr(ml_id, json_data):
+def _attach_model_xattr(ml_id, json_data, project_name):
     """
     Utility method for putting JSON data into elastic search
 
@@ -467,15 +494,31 @@ def _attach_model_xattr(ml_id, json_data):
         None
 
     """
+
     headers = {'Content-type': 'application/json'}
     resource_url = constants.DELIMITERS.SLASH_DELIMITER + \
                    constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER + \
                    constants.REST_CONFIG.HOPSWORKS_PROJECT_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER + \
                    hdfs.project_id() + constants.DELIMITERS.SLASH_DELIMITER + \
+                   constants.REST_CONFIG.HOPSWORKS_MODEL_REGISTRY_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER + \
+                   str(get_model_registry_id(project_name)) + constants.DELIMITERS.SLASH_DELIMITER + \
                    constants.REST_CONFIG.HOPSWORKS_MODELS_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER + \
                    ml_id
 
-    resp = util.send_request('PUT', resource_url, data=json_data, headers=headers)
+    if constants.ENV_VARIABLES.JOB_NAME_ENV_VAR in os.environ:
+        resource_url = resource_url + "?jobName=" + os.environ[constants.ENV_VARIABLES.JOB_NAME_ENV_VAR]
+    elif constants.ENV_VARIABLES.KERNEL_ID_ENV_VAR in os.environ:
+        resource_url = resource_url + "?kernelId=" + os.environ[constants.ENV_VARIABLES.KERNEL_ID_ENV_VAR]
+
+    response = util.send_request('PUT', resource_url, data=json_data, headers=headers)
+    response_object = response.json()
+    if response.status_code >= 400:
+        error_code, error_msg, user_msg = util._parse_rest_error(response_object)
+        raise RestAPIError("Could not attach model summary to model (url: {}), server response: \n "
+                           "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}"
+            .format(resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
+    else:
+        return response_object
 
 def _populate_experiment(model_name, function, type, hp, description, app_id, direction, optimization_key):
     """
